<?php
/**
	* DISCLAIMER
	*
	* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
	* versions in the future. If you wish to customize PrestaShop for your
	* needs please refer tohttp://www.prestashop.com for more information.
	* We offer the best and most useful modules PrestaShop and modifications for your online store.
	*
	* @category  PrestaShop Module
	* @author    knowband.com <support@knowband.com>
	* @copyright 2015 knowband
	* @license   see file: LICENSE.txt
	*
	* Description
	*
	* Controller file
	*/

if (!defined('_PS_VERSION_'))
	exit;

require_once dirname(__FILE__).'/classes/abandonedcart_core.php';

class Abandonedcart extends AbandonedCartCore
{

	private $shopping_settings = array();
	protected $admin_path;

	public function __construct()
	{
		$this->name = 'abandonedcart';
		$this->tab = 'checkout';
		$this->version = '1.0.0';
		$this->author = 'Knowband';
		$this->need_instance = 0;
		$this->ps_versions_compliancy = array('min' => '1.6.0.4', 'max' => _PS_VERSION_);
		$this->bootstrap = true;

		$this->displayName = $this->l('Free Abandoned Cart');
		$this->description = $this->l('This module will convert the abandoned carts into sales.');
		$this->confirmUninstall = $this->l('Are you sure you want to uninstall?');

		parent::__construct();
	}

	public function getErrors()
	{
		return $this->custom_errors;
	}

	public function install()
	{
		if (Shop::isFeatureActive())
			Shop::setContext(Shop::CONTEXT_ALL);

		if (!parent::install())
			return false;

		$this->installModel();

		$this->shopping_settings = $this->getDefaultSettings();

		Configuration::updateGlobalValue('VELSOF_ABANDONEDCART', serialize($this->shopping_settings));

		if (!Configuration::get('VELSOF_ABANDONEDCART_START_DATE'))
			Configuration::updateGlobalValue('VELSOF_ABANDONEDCART_START_DATE', date('Y-m-d H:i:s'));

		return true;
	}

	public function uninstall()
	{
		if (!parent::uninstall())
			return false;

		return true;
	}

	public function getContent()
	{
		if (Tools::isSubmit('ajax') && Tools::getValue('ajax'))
			$this->doAjaxProcess();

		$output = null;
		if (Tools::isSubmit('abd_configuration_form'))
		{
			$settings = Tools::getValue('velsof_abandoncart');
			if (!$settings || empty($settings))
				$output .= $this->displayError($this->l('Invalid Configuration value'));
			else
			{
				$tmp = $this->getDefaultSettings();
				$settings['plugin_id'] = $tmp['plugin_id'];
				Configuration::updateValue('VELSOF_ABANDONEDCART', serialize($settings));
				$output .= $this->displayConfirmation($this->l('The settings have been updated.'));
			}
		}
		else if ($cart_re_display_val = Tools::getValue('enable_cart_redisplay'))
		{
			if (Validate::isBool($cart_re_display_val) && Validate::isCleanHtml($cart_re_display_val))
			{
				Configuration::updateValue('PS_CART_FOLLOWING', $cart_re_display_val);
				$output .= $this->displayConfirmation($this->l('Cart Re-Display has been enabled.'));
			}
			else
				$output .= $this->displayError($this->l('Invalid Cart Re-Display Submission'));
		}

		if (!Configuration::get('VELSOF_ABANDONEDCART') || Configuration::get('VELSOF_ABANDONEDCART') == '')
			$settings = $this->getDefaultSettings();
		else
			$settings = unserialize(Configuration::get('VELSOF_ABANDONEDCART'));

		$this->shopping_settings = $settings;
		$this->updateAbandonList();

		$cron_link = $this->context->link->getModuleLink('abandonedcart', 'cron');
		$dot_found = 0;
		$needle = 'index.php';
		$dot_found = strpos($cron_link, $needle);
		if ($dot_found !== false)
			$ch = '&';
		else
			$ch = '?';

		$custom_ssl_var = 0;

		if (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == 'on')
			$custom_ssl_var = 1;

		if ((bool)Configuration::get('PS_SSL_ENABLED') && $custom_ssl_var == 1)
			$module_dir = _PS_BASE_URL_SSL_.__PS_BASE_URI__.str_replace(_PS_ROOT_DIR_.'/', '', _PS_MODULE_DIR_);
		else
			$module_dir = _PS_BASE_URL_.__PS_BASE_URI__.str_replace(_PS_ROOT_DIR_.'/', '', _PS_MODULE_DIR_);

		$this->smarty->assign(array(
			'cancel_action' => AdminController::$currentIndex.'&token='.Tools::getAdminTokenLite('AdminModules'),
			'action' => AdminController::$currentIndex.'&token='.Tools::getAdminTokenLite('AdminModules').'&configure='.$this->name,
			'velsof_abandoncart' => $settings,
			'languages' => Language::getLanguages(true),
			'default_language' => $this->context->language->id,
			'currency_format' => $this->context->currency->format,
			'currency_blank' => $this->context->currency->blank,
			'currency_sign' => $this->context->currency->sign,
			'cart_redisplay' => (Configuration::get('PS_CART_FOLLOWING')) ? true : false,
			'path' => _PS_BASE_URL_.__PS_BASE_URI__.str_replace(_PS_ROOT_DIR_.'/', '', _PS_MODULE_DIR_).$this->name.'/',
			'admin_path' => _PS_BASE_URL_.__PS_BASE_URI__.str_replace(_PS_ROOT_DIR_.'/', '', _PS_ADMIN_DIR_),
			'root_path' => _PS_BASE_URL_.__PS_BASE_URI__.str_replace(_PS_ROOT_DIR_.'/', '', ''),
			'abandon_list' => $this->getAbandonList(),
			'token' => Tools::getAdminTokenLite('AdminModules'),
			'front_cron_url' => $cron_link.$ch,
                        'image_path' => $module_dir.'abandonedcart/views/img/admin/'
		));

		$this->loadMedia();

		$output .= $this->display(__FILE__, 'views/templates/admin/abandonedcart.tpl');
		return $output;
	}

	private function doAjaxProcess()
	{
		$json = array();
		switch (Tools::getValue('method'))
		{
			case 'getabandonlist' :
				{
					$this->updateAbandonList();
					$json = $this->getAbandonList();
					break;
				}
			case 'getCustomerDetail' :
				{
					$json = $this->getCustomerDetail(Tools::getValue('id_customer'));
					break;
				}
			case 'getCustomerCartDetail' :
				{
					$json = $this->getCustomerCartDetail(Tools::getValue('id_customer'), Tools::getValue('id_cart'));
					break;
				}
			case 'deleteabandon' :
				{
					if ($this->deleteAbandonCart(Tools::getValue('id_abandon')))
						$json = array('status' => true, 'message' => $this->l('Requested abandon cart deleted successfully.'));
					else
						$json = array('status' => false, 'message' => $this->l('Not able to delete requested abandon cart.'));
					break;
				}
			case 'getdefaultlanguage' :
				{
					echo $this->context->language->id;
					die;
				}
		}

		header('Content-Type: application/json', true);
		echo Tools::jsonEncode($json);
		die;
	}

	private function getCustomerDetail($id_customer)
	{
		$data = array();
		$customer = new Customer($id_customer);
		$data = array(
			'id_customer' => $id_customer,
			'fname' => $customer->firstname,
			'lname' => $customer->lastname,
			'email' => $customer->email
		);
		return $data;
	}

	private function getCustomerCartDetail($id_customer, $id_cart)
	{
		$data = array('customer' => array(), 'cart_total' => 0, 'products' => array());
		$cart = new Cart($id_cart);
		$detail = $cart->getProducts();
		$link = new Link();
		if ($detail && count($detail) > 0)
		{
			foreach ($detail as $product)
			{
				if ((bool)Configuration::get('PS_SSL_ENABLED'))
					$img_path = 'https://'.$link->getImageLink($product['link_rewrite'], $product['id_image']);
				else
					$img_path = 'http://'.$link->getImageLink($product['link_rewrite'], $product['id_image']);

				$product['pro_link'] = $this->context->link->getProductLink($product['id_product']);
				$product['img_link'] = $img_path;
				$product['price_wt'] = Tools::displayPrice($product['price_wt']);
				$product['total_wt'] = Tools::displayPrice($product['total_wt']);
				$data['products'][] = $product;
			}
			$data['cart_total'] = Tools::displayPrice($cart->getordertotal());
		}

		$data['customer'] = $this->getCustomerDetail($id_customer);
		return $data;
	}

	public function deleteAbandonCart($id_abandon)
	{
		$sql = 'update '._DB_PREFIX_.self::ABANDON_TABLE_NAME.' set shows= "0" where id_abandon='.(int)$id_abandon;
		if (Db::getInstance(_PS_USE_SQL_SLAVE_)->execute($sql))
			return true;
		else
			return false;
	}

	public function updateAbandonList($cron = false)
	{
		/*
			* Check the guest cart which are turned into registed user cart later
			*/
		$qry = 'Select * from '._DB_PREFIX_.parent::ABANDON_TABLE_NAME.' where id_customer = 0';
		$guest_carts = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($qry);
		if (is_array($guest_carts) && count($guest_carts) > 0)
		{
			foreach ($guest_carts as $c)
			{
				$t = 'Select id_customer from '._DB_PREFIX_.'cart 
					where id_cart = '.(int)$c['id_cart'].' AND id_customer > 0';
				if ($id_customer = Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue($t))
				{
					$check_guest = 'select is_guest from '._DB_PREFIX_.'customer where id_customer = '.(int)$id_customer;
					$is_guest = Db::getInstance(_PS_USE_SQL_SLAVE_)->getRow($check_guest);
					$up_qry = 'Update '._DB_PREFIX_.parent::ABANDON_TABLE_NAME
						.' set id_customer = '.(int)$id_customer.', is_guest = "'.(int)$is_guest['is_guest'].'"
						WHERE id_cart = '.(int)$c['id_cart'].' AND id_abandon = '.(int)$c['id_abandon'];
					Db::getInstance(_PS_USE_SQL_SLAVE_)->execute($up_qry);
				}
			}
		}

		$configurations = unserialize(Configuration::get('VELSOF_ABANDONEDCART'));
		if ($configurations['enable'] != 1)
			return false;

//		if (!Configuration::get('PS_CART_FOLLOWING'))
//			return false;

		$velsof_abandoncart_start_date = Configuration::get('VELSOF_ABANDONEDCART_START_DATE');

		$delay = (int)$configurations['delay_hours'] + (24 * (int)$configurations['delay_days']);
		$delay_time = date('Y-m-d H:i:s', strtotime('-'.$delay.' hours'));

		$sql = 'select c.* from '._DB_PREFIX_.'cart as c 
			WHERE c.date_upd >= "'.pSQL(date('Y-m-d H:i:s', strtotime($velsof_abandoncart_start_date))).'"';

		$carts = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);
		$carts_added = 0;
		if (is_array($carts) && count($carts) > 0)
		{
			foreach ($carts as $cart)
			{
				$check_order_sql = 'Select count(*) as row from '._DB_PREFIX_.'orders WHERE id_cart = '.(int)$cart['id_cart'];
				$check_order = Db::getInstance(_PS_USE_SQL_SLAVE_)->getRow($check_order_sql);
				if (!is_array($check_order) || $check_order['row'] <= 0)
				{
					$consider_abandon_sql = 'Select count(*) as row from '._DB_PREFIX_.'cart 
						WHERE id_cart = '.(int)$cart['id_cart'].' AND date_upd <= "'.pSQL($delay_time).'"';
					$consider_abandon = Db::getInstance(_PS_USE_SQL_SLAVE_)->getRow($consider_abandon_sql);
					if (is_array($consider_abandon) && $consider_abandon['row'] > 0)
					{
						$is_exist_sql = 'Select count(*) as row from '._DB_PREFIX_.parent::ABANDON_TABLE_NAME.' where id_cart ='.(int)$cart['id_cart'];
						$is_exist = Db::getInstance(_PS_USE_SQL_SLAVE_)->getRow($is_exist_sql);
						if (!is_array($is_exist) || $is_exist['row'] <= 0)
						{
							$customer = new Customer((int)$cart['id_customer']);
							$insert_abandon_data = 'INSERT into '._DB_PREFIX_.parent::ABANDON_TABLE_NAME
								.' (id_cart,id_shop,id_lang,id_customer,is_guest,date_add,date_upd) values ('
								.(int)$cart['id_cart'].', '
								.(int)$cart['id_shop'].', '
								.(int)$cart['id_lang'].', '
								.(int)$cart['id_customer'].', "'.(int)$customer->isGuest().'", "'
								.pSQL($cart['date_add']).'", "'.pSQL($cart['date_upd']).'")';

							Db::getInstance(_PS_USE_SQL_SLAVE_)->execute($insert_abandon_data);
							$carts_added++;
						}
						else
						{
							$update_date_update = 'update '._DB_PREFIX_.parent::ABANDON_TABLE_NAME.' set
							date_upd = "'.pSQL($cart['date_upd']).'"  where id_cart = '.(int)$cart['id_cart'];

							Db::getInstance(_PS_USE_SQL_SLAVE_)->execute($update_date_update);
						}
					}
				}
			}
		}
		if ($cron)
		{
			echo '<div class="text_to_show" style="font-size:16px;padding: 10px;">'.$this->l('Total').' '
				.$carts_added.' '.$this->l('new Carts added.').'</div>';
			die;
		}
	}



}
