<?php
/**
	* DISCLAIMER
	*
	* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
	* versions in the future. If you wish to customize PrestaShop for your
	* needs please refer tohttp://www.prestashop.com for more information.
	* We offer the best and most useful modules PrestaShop and modifications for your online store.
	*
	* @category  PrestaShop Module
	* @author    knowband.com <support@knowband.com>
	* @copyright 2015 knowband
	* @license   see file: LICENSE.txt
	*/

class AbandonedCartCore extends Module
{
	const ABANDON_TABLE_NAME = 'velsof_abd_cart';
	const DATE_FORMAT = 'Y-m-d';
	const ITEM_PER_PAGE = 10;

	/*
		* use "left" to display pagination on left side
		*/
	const PAGINATION_ALIGN = 'right';

	public function __construct()
	{
		parent::__construct();

		if (!Configuration::get('VELSOF_ABANDONEDCART'))
			$this->warning = $this->l('No name provided');
	}

	public function install()
	{
		if (!parent::install())
			return false;
		return true;
	}

	public function uninstall()
	{
		if (!parent::uninstall()
			|| !Configuration::deleteByName('VELSOF_ABANDONEDCART'))
			return false;

		return true;
	}

	protected function installModel()
	{
		$abandon_cart_table = 'CREATE TABLE IF NOT EXISTS '._DB_PREFIX_.self::ABANDON_TABLE_NAME.' (
			id_abandon int(11) NOT NULL AUTO_INCREMENT PRIMARY KEY,
			id_cart int(10) unsigned NOT NULL UNIQUE,
			id_shop int(11) DEFAULT 0,
			id_lang int(11) DEFAULT 0,
			is_converted enum("1","0") NULL DEFAULT "0",
			id_customer int(10) unsigned NULL,
			auto_email enum("1","0") NULL DEFAULT "1",
			is_guest enum("1","0"),
			shows enum("1","0") NULL DEFAULT "1",
			reminder_sent enum("1","0") NULL DEFAULT "0",
			date_add datetime NULL,
			date_upd datetime NULL,
			INDEX (`id_cart`, `id_customer`),
			FOREIGN KEY (id_cart) references '._DB_PREFIX_.'cart (id_cart) ON DELETE CASCADE			
			) CHARACTER SET utf8 COLLATE utf8_general_ci';

		Db::getInstance(_PS_USE_SQL_SLAVE_)->execute($abandon_cart_table);
	}

	public function getDefaultSettings()
	{
		$settings = array(
			'plugin_id' => 'PS0008',
			'enable' => 1,
			'enable_test' => 0,
			'testing_email_id' => '',
			'delay_days' => 1,
			'delay_hours' => 0,
			'schedule' => 1
		);
		return $settings;
	}

	protected function loadMedia()
	{
		$dir = 'views/css/';
		$this->context->controller->addCSS($this->_path.$dir.'admin_abandon.css');
		$this->context->controller->addCSS($this->_path.$dir.'bootstrap.css');
		$this->context->controller->addCSS($this->_path.$dir.'responsive.css');
		$this->context->controller->addCSS($this->_path.$dir.'select2.css');
		$this->context->controller->addCSS($this->_path.$dir.'default.css');
		$this->context->controller->addCSS($this->_path.$dir.'jquery.easy-pie-chart.css');
		$this->context->controller->addCSS($this->_path.$dir.'DT_bootstrap.css');
		//$this->context->controller->addCSS($this->_path.'css/style.css');
		$this->context->controller->addCSS($this->_path.$dir.'fonts/glyphicons_regular.css');
		$this->context->controller->addCSS($this->_path.$dir.'bootstrap-switch.css');
		$this->context->controller->addCSS($this->_path.$dir.'style-light.css');

		$dir = 'views/js/';
		$this->context->controller->addJs($this->_path.$dir.'admin_abandoncart.js');
		$this->context->controller->addJs($this->_path.$dir.'table/bootbox.js');
		$this->context->controller->addJs($this->_path.$dir.'uniform/jquery.uniform.min.js');
		$this->context->controller->addJs($this->_path.$dir.'bootstrap-switch.js');
		//$this->context->controller->addJs($this->_path.$dir.'table/tables.js');
		$this->context->controller->addJs($this->_path.$dir.'table/jquery.dataTables.js');
		$this->context->controller->addJs($this->_path.$dir.'table/DT_bootstrap.js');
		$this->context->controller->addJs($this->_path.$dir.'common.js');
	}

	protected function getAbandonList()
	{
		$page_number = 1;
		$item_per_page = self::ITEM_PER_PAGE;
		if (Tools::getValue('page_number') && Tools::getValue('page_number') > 1)
			$page_number = (int)Tools::getValue('page_number');

		if (Tools::getValue('cart_per_page') && Tools::getValue('cart_per_page') > 1)
			$item_per_page = (int)Tools::getValue('cart_per_page');

		$abandon_cart_query = 'select {COLUMN} from '._DB_PREFIX_.self::ABANDON_TABLE_NAME.
			' as abd INNER JOIN '._DB_PREFIX_.'cart_product as cp on (abd.id_cart = cp.id_cart) 
			LEFT JOIN '._DB_PREFIX_.'customer as c on (abd.id_customer = c.id_customer) 
			where (abd.is_converted = "0" AND abd.shows = "1")';

		if (Tools::getIsset('email') && trim(Tools::getValue('email')) != '')
		{
			if (Tools::getIsset('qtype') && Tools::getValue('qtype') == 1)
				$abandon_cart_query .= ' AND c.email = "'.pSQL(trim(Tools::getValue('email'))).'"';
			else if (Tools::getIsset('qtype') && Tools::getValue('qtype') == 2)
				$abandon_cart_query .= ' AND c.email LIKE "%'.pSQL(trim(Tools::getValue('email'))).'%"';
		}

		if (Tools::getIsset('ctype') && trim(Tools::getValue('ctype')) == 1)
			$abandon_cart_query .= ' AND c.is_guest = 0';
		else if (Tools::getIsset('ctype') && trim(Tools::getValue('ctype')) == 2)
			$abandon_cart_query .= ' AND abd.id_customer = 0';
		else if (Tools::getIsset('ctype') && trim(Tools::getValue('ctype')) == 3)
			$abandon_cart_query .= ' AND abd.is_guest = 1 AND abd.id_customer > 0';

		$settings = unserialize(Configuration::get('VELSOF_ABANDONEDCART'));
		$total_delay = $settings['delay_hours'] + ( 24 * $settings['delay_days']);
		$delay_date = date('Y-m-d H:i:s', strtotime('-'.$total_delay.' hours'));
		$abandon_cart_query .= ' AND abd.id_shop = '.(int)$this->context->shop->id.
			' AND abd.date_upd < "'.pSQL($delay_date).'"';

		$abandon_cart_query .= ' GROUP BY cp.id_cart HAVING SUM(cp.quantity) > 0 ORDER BY abd.date_upd DESC';

		$total_records = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS(str_replace('{COLUMN}', 'count(*) as total', $abandon_cart_query));

		if (count($total_records) <= 0)
			return array('flag' => false, 'pagination' => '');

		$total_records = count($total_records);
		$total_pages = ceil((int)$total_records / $item_per_page);

		$page_position = (($page_number - 1) * $item_per_page);

		$abandon_cart_query .= ' LIMIT '.(int)$page_position.', '.(int)$item_per_page;

		$columns = 'abd.*, c.email, c.is_guest, c.firstname, c.lastname';

		$results = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS(str_replace('{COLUMN}', $columns, $abandon_cart_query));
		if ($results && count($results) > 0)
		{
			$abandon_customers = array();
			foreach ($results as $list)
			{
				if (empty($list['id_customer']) || $list['id_customer'] == 0)
				{
					$list['firstname'] = 'ABC';
					$list['lastname'] = 'DEF';
					$list['email'] = 'noreply@email.com';
					$list['is_guest'] = true;
				}

				$list['date_add'] = Tools::displayDate($list['date_add'], null, true);
				$list['date_upd'] = Tools::displayDate($list['date_upd'], null, true);

				$check_coupon = 'select COUNT(cr.id_cart_rule) as count
					from '._DB_PREFIX_.'cart_rule as cr INNER JOIN '._DB_PREFIX_.'cart_rule_lang as crl 
					on (cr.id_cart_rule = crl.id_cart_rule)
					where cr.id_customer = '.(int)$list['id_customer'].' AND crl.name = "ABD['.(string)$list['email'].']" 
					AND crl.id_lang = '.(int)$list['id_lang'];

				$has_coupon = Db::getInstance(_PS_USE_SQL_SLAVE_)->getRow($check_coupon);

				if ($has_coupon && $has_coupon['count'] > 0)
					$list['has_coupon'] = true;
				else
					$list['has_coupon'] = false;

				$lang = Language::getLanguage((int)$list['id_lang']);
				$list['language_text'] = $lang['name'];
				$abandon_customers[] = $list;
			}
			$paging = $this->customPaginator(
					$page_number,
					$total_records,
					$total_pages,
					'getAbandonedList');
			return array(
				'flag' => true,
				'data' => $abandon_customers,
				'pagination' => $paging['paging'],
				'start_serial' => $paging['serial']
			);
		}
		else
			return array('flag' => false, 'pagination' => '');
	}

	protected function customPaginator($current_page = 1, $total_records, $total_pages, $ajaxcallfn = '')
	{
		$summary_txt = '';
		$pagination = '';
		if ($total_pages > 0 && $total_pages != 1 && $current_page <= $total_pages)
		{ //verify total pages and current page number
			$summary_align = 'abd-pagination-left';
			$pagination_align = 'abd-pagination-left';
			if (self::PAGINATION_ALIGN == 'right')
			{
				$summary_align = 'abd-pagination-left';
				$pagination_align = 'abd-pagination-right';
			}
			$record_start = $current_page;
			$record_end = self::ITEM_PER_PAGE;
			if ($current_page > 1)
			{
				$record_start = (($current_page - 1) * self::ITEM_PER_PAGE) + 1;
				if ($current_page == $total_pages)
					$record_end = $total_records;
				else
					$record_end = $current_page * self::ITEM_PER_PAGE;
			}

			$summary_txt = '<div class="'.$summary_align.' abd-paginate-summary">
				Showing '.$record_start.' to '.$record_end.' of '.$total_records.' ('.$total_pages.' pages)</div>';

			$pagination .= '<div class="'.$pagination_align.'"><ul class="abd-pagination">';

			$ajax_call_function = '';
			if ($ajaxcallfn != '')
				$ajax_call_function .= $ajaxcallfn.'({page_number});';

			$right_links = $current_page + 3;
			$previous = $current_page - 3; //previous link
			$first_link = true; //boolean var to decide our first link

			if ($current_page > 1)
			{
				$previous_link = ($previous == 0) ? 1 : $previous;
				$pagination .= '<li class="first"><a href="javascript:void(0)" data-page="1" 
					onclick="'.str_replace('{page_number}', 1, $ajax_call_function).'" 
					title="First">&laquo;</a></li>'; //first link
				$pagination .= '<li><a href="javascript:void(0)" data-page="'.$previous_link.'" 
					onclick="'.str_replace('{page_number}', $previous_link, $ajax_call_function).'" 
					title="Previous">&lt;</a></li>'; //previous link
				for ($i = ($current_page - 2); $i < $current_page; $i++)
				{
					if ($i > 0)
						$pagination .= '<li><a href="javascript:void(0)" data-page="'.$i.'" 
						onclick="'.str_replace('{page_number}', $i, $ajax_call_function).'" 
						title="Page'.$i.'">'.$i.'</a></li>';
				}
				$first_link = false; //set first link to false
			}

			if ($first_link)
				$pagination .= '<li class="first active">'.$current_page.'</li>';
			elseif ($current_page == $total_pages)
				$pagination .= '<li class="last active">'.$current_page.'</li>';
			else
				$pagination .= '<li class="active">'.$current_page.'</li>';

			for ($i = $current_page + 1; $i < $right_links; $i++)
			{
				if ($i <= $total_pages)
					$pagination .= '<li><a href="javascript:void(0)" data-page="'.$i.'" 
					onclick="'.str_replace('{page_number}', $i, $ajax_call_function).'" 
					title="Page '.$i.'">'.$i.'</a></li>';
			}
			if ($current_page < $total_pages)
			{
				$next_link = ($i > $total_pages) ? $total_pages : $i;
				$pagination .= '<li><a href="javascript:void(0)" data-page="'.$next_link.'" 
					onclick="'.str_replace('{page_number}', $next_link, $ajax_call_function).'" 
					title="Next">&gt;</a></li>'; //next link
				$pagination .= '<li class="last"><a href="javascript:void(0)" data-page="'.$total_pages.'" 
					onclick="'.str_replace('{page_number}', $total_pages, $ajax_call_function).'" 
					title="Last">&raquo;</a></li>'; //last link
			}

			$pagination .= '</div></ul>';
			return array('paging' => $summary_txt.$pagination, 'serial' => $record_start);
		}
		return array('paging' => '', 'serial' => 1);
	}
}

?>